Method: MatrixSdk::Protocols::CS#get_room_encryption_settings

Defined in:
lib/matrix_sdk/protocols/cs.rb

#get_room_encryption_settings(room_id, **params) ⇒ Response

Gets the encryption configuration for a room

Parameters:

  • room_id (MXID, String)

    The room ID to look up

  • params (Hash)

    Extra options to provide to the request, see #get_room_state

Returns:

  • (Response)

    A response hash with the configuration the room was created for

See Also:



1065
1066
1067
# File 'lib/matrix_sdk/protocols/cs.rb', line 1065

def get_room_encryption_settings(room_id, **params)
  get_room_state(room_id, 'm.room.encryption', **params)
end