Class: MatrixSdk::Room
Overview
A class for tracking the information about a room on Matrix
Direct Known Subclasses
Constant Summary
Constants included from Util::Tinycache
Instance Attribute Summary collapse
-
#client ⇒ Client
readonly
The client for the room.
-
#event_history_limit ⇒ Fixnum
The limit of events to keep in the event log.
-
#events ⇒ Object
readonly
Returns the value of attribute events.
-
#id ⇒ String
(also: #room_id)
readonly
The internal ID of the room.
- #on_account_data ⇒ Object readonly
- #on_ephemeral_event ⇒ Object readonly
- #on_event ⇒ Object readonly
- #on_state_event ⇒ Object readonly
Instance Method Summary collapse
- #account_data ⇒ Object
-
#add_alias(room_alias) ⇒ Boolean
Add an alias to the room.
-
#add_tag(tag, **data) ⇒ Object
Add a tag to the room.
-
#admin!(user, level: 100) ⇒ Object
Make a user an admin in the room.
-
#admin?(user, target_level: 100) ⇒ Boolean
Check if a user is an admin in the room.
-
#aliases(canonical_only: true) ⇒ Array[String]
Gets the room aliases.
-
#all_members(**params) ⇒ Array(User)
Get all members (member events) in the room.
-
#allow_guests=(allow_guests) ⇒ Object
Sets if guests are allowed in the room.
-
#avatar_url ⇒ String?
Gets the avatar url of the room - if any.
-
#avatar_url=(avatar_url) ⇒ Object
Sets a new avatar URL for the room.
-
#backfill_messages(*args, reverse: false, limit: 10) ⇒ Object
Backfills messages into the room history.
-
#ban_user(user_id, reason = '') ⇒ Boolean
Bans a user from the room.
-
#canonical_alias ⇒ String?
The canonical alias of the room.
-
#creation_info ⇒ Response
Gets the room creation information.
-
#display_name ⇒ String
Gets a human-readable name for the room.
-
#dm=(direct) ⇒ Object
Mark a room as a direct (1:1) message Room.
-
#dm?(members_only: false) ⇒ Boolean
Checks if the room is a direct message / 1:1 room.
-
#get_account_data(type) ⇒ Hash
Retrieves a custom entry from the room-specific account data.
-
#guest_access ⇒ :can_join, :forbidden
Gets the guest access rights for the room.
-
#guest_access=(guest_access) ⇒ Object
Sets the guest access status for the room.
-
#guest_access? ⇒ Boolean
Checks if
guest_accessis set to:can_join. -
#history_visibility ⇒ :invited, ...
Gets the history visibility of the room.
-
#initialize(client, room_id, data = {}) ⇒ Room
constructor
Create a new room instance.
-
#inspect ⇒ String
An inspect method that skips a handful of instance variables to avoid flooding the terminal with debug data.
-
#invite_only=(invite_only) ⇒ Object
Sets if the room should be invite only or not.
-
#invite_only? ⇒ Boolean
Checks if
join_ruleis set to:invite. -
#invite_user(user_id) ⇒ Boolean
Invites a user into the room.
-
#join_rule ⇒ :public, ...
Gets the join rule for the room.
-
#join_rule=(join_rule) ⇒ Object
Sets the join rule of the room.
-
#joined_members ⇒ Array(User)
Populates and returns the #members array.
-
#kick_user(user_id, reason = '') ⇒ Boolean
Kicks a user from the room.
-
#knock_only? ⇒ Boolean
Checks if
join_ruleis set to:knock. -
#leave ⇒ Boolean
Requests to be removed from the room.
- #members ⇒ Object
-
#moderator!(user, level: 50) ⇒ Object
Make a user a moderator in the room.
-
#moderator?(user, target_level: 50) ⇒ Boolean
Check if a user is a moderator in the room.
-
#modify_required_power_levels(events = nil, params = {}) ⇒ Boolean
Modifies the required power levels for actions in the room.
-
#modify_user_power_levels(users = nil, users_default = nil) ⇒ Boolean
Modifies the power levels of the room.
-
#name ⇒ String?
Gets the current name of the room, querying the API if necessary.
-
#name=(name) ⇒ Object
Sets a new name on the room.
-
#power_levels ⇒ Hash
Get the power levels of the room.
-
#redact_message(event_id, reason = nil) ⇒ Object
Redacts a message from the room.
-
#reload! ⇒ Object
(also: #refresh!)
Refreshes the room state caches for name, topic, and aliases.
-
#reload_aliases! ⇒ Boolean
(also: #refresh_aliases!)
Reloads the list of aliases by an API query.
-
#reload_name! ⇒ Boolean
(also: #refresh_name!)
Reloads the name of the room.
-
#reload_topic! ⇒ Boolean
(also: #refresh_topic!)
Reloads the topic of the room.
-
#remove_tag(tag) ⇒ Object
Remove a tag from the room.
-
#report_message(event_id, reason:, score: -100)) ⇒ Object
Reports a message in the room.
-
#room_type ⇒ 'm.space', ...
Retrieves the type of the room.
-
#room_version ⇒ String
Retrieves the room version.
-
#send_audio(url, name, audio_info = {}) ⇒ Object
Sends a link to an audio clip to the room.
-
#send_custom_message(body, content = {}, msgtype: nil) ⇒ Object
Sends a customized message to the Room.
-
#send_emote(text) ⇒ Object
Sends an emote (/me) message to the room.
-
#send_event(type, content = {}) ⇒ Object
Sends a custom timeline event to the Room.
-
#send_file(url, name, file_info = {}) ⇒ Object
Sends a link to a generic file to the room.
-
#send_html(html, body = nil, msgtype: nil, format: nil) ⇒ Object
Sends a custom HTML message to the room.
-
#send_image(url, name, image_info = {}) ⇒ Object
Sends a link to an image to the room.
-
#send_location(geo_uri, name, thumbnail_url = nil, thumbnail_info = {}) ⇒ Object
Sends a location object to the room.
-
#send_notice(text) ⇒ Object
Sends a notice (bot) message to the room.
-
#send_text(text) ⇒ Object
Sends a plain-text message to the room.
-
#send_video(url, name, video_info = {}) ⇒ Object
Sends a link to a video to the room.
-
#set_account_data(type, account_data) ⇒ Object
Stores a custom entry into the room-specific account data.
-
#set_user_profile(display_name: nil, avatar_url: nil, reason: nil) ⇒ Object
Changes the room-specific user profile.
-
#space? ⇒ Boolean?
Checks if the room is a Matrix Space.
-
#tags ⇒ Response
Returns a list of the room tags.
- #to_s ⇒ Object
-
#to_space ⇒ Object
Casting operators.
-
#topic ⇒ String?
Gets the room topic - if any.
-
#topic=(topic) ⇒ Object
Sets a new topic on the room.
-
#unban_user(user_id) ⇒ Boolean
Unbans a user from the room.
-
#user_can_send?(user, event, state: false) ⇒ Boolean
Checks if a user can send a given event type in the room.
-
#user_powerlevel(user, use_default: true) ⇒ Integer?
Gets the power level of a user in the room.
-
#world_readable? ⇒ Boolean
(also: #world_readable)
Checks if the room history is world readable.
Methods included from Extensions
Methods included from Util::Tinycache
adapter, adapter=, cached, extended, tinycache_adapter_config
Methods included from Logging
Constructor Details
#initialize(client, room_id, data = {}) ⇒ Room
This method isn’t supposed to be used directly, rather rooms should be retrieved from the Client abstraction.
Create a new room instance
67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 |
# File 'lib/matrix_sdk/room.rb', line 67 def initialize(client, room_id, data = {}) if client.is_a? Room copy = client client = copy.client room_id = copy.id # data = copy.attributes end raise ArgumentError, 'Must be given a Client instance' unless client.is_a? Client @client = client room_id = MXID.new room_id unless room_id.is_a?(MXID) raise ArgumentError, 'room_id must be a valid Room ID' unless room_id.room_id? @events = [] @event_history_limit = 10 @room_type = nil @prev_batch = nil data.each do |k, v| next if %i[client].include? k if respond_to?("#{k}_cached?".to_sym) && send("#{k}_cached?".to_sym) tinycache_adapter.write(k, v) elsif instance_variable_defined? "@#{k}" instance_variable_set("@#{k}", v) end end @id = room_id.to_s logger.debug "Created room #{room_id}" end |
Instance Attribute Details
#client ⇒ Client (readonly)
Returns the client for the room.
25 |
# File 'lib/matrix_sdk/room.rb', line 25 attr_reader :id, :client, :events |
#event_history_limit ⇒ Fixnum
Returns the limit of events to keep in the event log.
16 17 18 |
# File 'lib/matrix_sdk/room.rb', line 16 def event_history_limit @event_history_limit end |
#events ⇒ Object (readonly)
Returns the value of attribute events.
25 |
# File 'lib/matrix_sdk/room.rb', line 25 attr_reader :id, :client, :events |
#id ⇒ String (readonly) Also known as: room_id
Returns the internal ID of the room.
25 26 27 |
# File 'lib/matrix_sdk/room.rb', line 25 def id @id end |
#on_account_data ⇒ Object (readonly)
132 133 134 |
# File 'lib/matrix_sdk/room.rb', line 132 def on_account_data ensure_room_handlers[:account_data] end |
#on_ephemeral_event ⇒ Object (readonly)
144 145 146 |
# File 'lib/matrix_sdk/room.rb', line 144 def on_ephemeral_event ensure_room_handlers[:ephemeral_event] end |
#on_event ⇒ Object (readonly)
126 127 128 |
# File 'lib/matrix_sdk/room.rb', line 126 def on_event ensure_room_handlers[:event] end |
#on_state_event ⇒ Object (readonly)
138 139 140 |
# File 'lib/matrix_sdk/room.rb', line 138 def on_state_event ensure_room_handlers[:state_event] end |
Instance Method Details
#account_data ⇒ Object
556 557 558 559 560 |
# File 'lib/matrix_sdk/room.rb', line 556 def account_data return MatrixSdk::Util::AccountDataCache.new client, room: self if client.cache == :none @account_data ||= MatrixSdk::Util::AccountDataCache.new client, room: self end |
#add_alias(room_alias) ⇒ Boolean
Add an alias to the room
727 728 729 730 731 |
# File 'lib/matrix_sdk/room.rb', line 727 def add_alias(room_alias) client.api.set_room_alias(id, room_alias) tinycache_adapter.read(:aliases) << room_alias if tinycache_adapter.exist?(:aliases) true end |
#add_tag(tag, **data) ⇒ Object
Add a tag to the room
672 673 674 675 |
# File 'lib/matrix_sdk/room.rb', line 672 def add_tag(tag, **data) client.api.add_user_tag(client.mxid, id, tag, data) true end |
#admin!(user, level: 100) ⇒ Object
Make a user an admin in the room
846 847 848 849 850 851 852 853 854 |
# File 'lib/matrix_sdk/room.rb', line 846 def admin!(user, level: 100) return true if admin?(user, target_level: level) user = user.id if user.is_a? User user = MXID.new(user.to_s) unless user.is_a? MXID raise ArgumentError, 'Must provide a valid user or MXID' unless user.user? modify_user_power_levels({ user.to_s.to_sym => level }) end |
#admin?(user, target_level: 100) ⇒ Boolean
Check if a user is an admin in the room
834 835 836 837 838 839 |
# File 'lib/matrix_sdk/room.rb', line 834 def admin?(user, target_level: 100) level = user_powerlevel(user, use_default: false) return false unless level level >= target_level end |
#aliases(canonical_only: true) ⇒ Array[String]
Gets the room aliases
311 312 313 314 315 316 317 |
# File 'lib/matrix_sdk/room.rb', line 311 def aliases(canonical_only: true) canonical = client.api.get_room_state(id, 'm.room.canonical_alias') rescue {} aliases = ([canonical[:alias]].compact + (canonical[:alt_aliases] || [])).uniq.sort return aliases if canonical_only (aliases + client.api.get_room_aliases(id).aliases).uniq.sort end |
#all_members(**params) ⇒ Array(User)
This will also count members who’ve knocked, been invited, have left, or have been banned.
Get all members (member events) in the room
201 202 203 |
# File 'lib/matrix_sdk/room.rb', line 201 def all_members(**params) client.api.get_room_members(id, **params)[:chunk].map { |ch| client.get_user(ch[:state_key]) } end |
#allow_guests=(allow_guests) ⇒ Object
Sets if guests are allowed in the room
763 764 765 766 |
# File 'lib/matrix_sdk/room.rb', line 763 def allow_guests=(allow_guests) self.guest_access = (allow_guests ? :can_join : :forbidden) allow_guests end |
#avatar_url ⇒ String?
Gets the avatar url of the room - if any
246 247 248 249 250 251 |
# File 'lib/matrix_sdk/room.rb', line 246 def avatar_url client.api.get_room_avatar(id)[:url] rescue MatrixNotFoundError # No avatar has been set nil end |
#avatar_url=(avatar_url) ⇒ Object
Sets a new avatar URL for the room
780 781 782 783 784 785 786 787 |
# File 'lib/matrix_sdk/room.rb', line 780 def avatar_url=(avatar_url) avatar_url = URI(avatar_url) unless avatar_url.is_a? URI raise ArgumentError, 'Must be a valid MXC URL' unless avatar_url.is_a? URI::MXC client.api.set_room_avatar(id, avatar_url) tinycache_adapter.write(:avatar_url, avatar_url) avatar_url end |
#backfill_messages(*args, reverse: false, limit: 10) ⇒ Object
This will trigger the on_event events as messages are added
Backfills messages into the room history
484 485 486 487 488 489 490 491 492 493 494 495 496 497 498 499 |
# File 'lib/matrix_sdk/room.rb', line 484 def (*args, reverse: false, limit: 10) # To be backwards-compatible if args.length == 2 reverse = args.first limit = args.last end data = client.api.(id, @prev_batch, direction: :b, limit: limit) events = data[:chunk] events.reverse! unless reverse events.each do |ev| put_event(ev) end true end |
#ban_user(user_id, reason = '') ⇒ Boolean
Bans a user from the room
531 532 533 534 535 |
# File 'lib/matrix_sdk/room.rb', line 531 def ban_user(user_id, reason = '') user_id = user_id.id if user_id.is_a? MatrixSdk::User client.api.ban_user(id, user_id, reason: reason) true end |
#canonical_alias ⇒ String?
Returns the canonical alias of the room.
177 178 179 180 181 |
# File 'lib/matrix_sdk/room.rb', line 177 def canonical_alias client.api.get_room_state(id, 'm.room.canonical_alias')[:alias] rescue MatrixSdk::MatrixNotFoundError nil end |
#creation_info ⇒ Response
Gets the room creation information
600 601 602 603 |
# File 'lib/matrix_sdk/room.rb', line 600 def creation_info # Not caching here, easier to cache the important values separately instead client.api.get_room_creation_info(id) end |
#display_name ⇒ String
This method will populate the #members list if it has to fall back to the member name generation.
Gets a human-readable name for the room
This will return #name or #canonical_alias if they’ve been set, otherwise it will query the API for members and generate a string from a subset of their names.
161 162 163 164 165 166 167 168 169 170 171 172 173 174 |
# File 'lib/matrix_sdk/room.rb', line 161 def display_name return name if name return canonical_alias if canonical_alias members = joined_members .reject { |m| m.user_id == client.mxid } .map(&:display_name) return members.first if members.one? return "#{members.first} and #{members.last}" if members.count == 2 return "#{members.first} and #{members.count - 1} others" if members.count > 2 'Empty Room' end |
#dm=(direct) ⇒ Object
Mark a room as a direct (1:1) message Room
228 229 230 231 232 233 234 235 236 237 238 239 240 241 |
# File 'lib/matrix_sdk/room.rb', line 228 def dm=(direct) rooms = client.direct_rooms dirty = false list_for_room = (rooms[id.to_s] ||= []) if direct && !list_for_room.include?(id.to_s) list_for_room << id.to_s dirty = true elsif !direct && list_for_room.include?(id.to_s) list_for_room.delete id.to_s rooms.delete id.to_s if list_for_room.empty? dirty = true end client.account_data['m.direct'] = rooms if dirty end |
#dm?(members_only: false) ⇒ Boolean
Checks if the room is a direct message / 1:1 room
221 222 223 224 225 |
# File 'lib/matrix_sdk/room.rb', line 221 def dm?(members_only: false) return true if !members_only && client.direct_rooms.any? { |_uid, rooms| rooms.include? id.to_s } joined_members.count <= 2 end |
#get_account_data(type) ⇒ Hash
Retrieves a custom entry from the room-specific account data
566 567 568 |
# File 'lib/matrix_sdk/room.rb', line 566 def get_account_data(type) account_data[type] end |
#guest_access ⇒ :can_join, :forbidden
Gets the guest access rights for the room
266 267 268 |
# File 'lib/matrix_sdk/room.rb', line 266 def guest_access client.api.get_room_guest_access(id)[:guest_access]&.to_sym end |
#guest_access=(guest_access) ⇒ Object
Sets the guest access status for the room
771 772 773 774 775 |
# File 'lib/matrix_sdk/room.rb', line 771 def guest_access=(guest_access) client.api.set_room_guest_access(id, guest_access) tinycache_adapter.write(:guest_access, guest_access) guest_access end |
#guest_access? ⇒ Boolean
Checks if guest_access is set to :can_join
278 279 280 |
# File 'lib/matrix_sdk/room.rb', line 278 def guest_access? guest_access == :can_join end |
#history_visibility ⇒ :invited, ...
Gets the history visibility of the room
295 296 297 |
# File 'lib/matrix_sdk/room.rb', line 295 def history_visibility client.api.get_room_state(id, 'm.room.history_visibility')[:history_visibility]&.to_sym end |
#inspect ⇒ String
An inspect method that skips a handful of instance variables to avoid flooding the terminal with debug data.
31 |
# File 'lib/matrix_sdk/room.rb', line 31 ignore_inspect :client, :events, :prev_batch, :logger, :tinycache_adapter |
#invite_only=(invite_only) ⇒ Object
Sets if the room should be invite only or not
746 747 748 749 |
# File 'lib/matrix_sdk/room.rb', line 746 def invite_only=(invite_only) self.join_rule = invite_only ? :invite : :public invite_only end |
#invite_only? ⇒ Boolean
Checks if join_rule is set to :invite
283 284 285 |
# File 'lib/matrix_sdk/room.rb', line 283 def invite_only? join_rule == :invite end |
#invite_user(user_id) ⇒ Boolean
Invites a user into the room
509 510 511 512 513 |
# File 'lib/matrix_sdk/room.rb', line 509 def invite_user(user_id) user_id = user_id.id if user_id.is_a? MatrixSdk::User client.api.invite_user(id, user_id) true end |
#join_rule ⇒ :public, ...
Gets the join rule for the room
273 274 275 |
# File 'lib/matrix_sdk/room.rb', line 273 def join_rule client.api.get_room_join_rules(id)[:join_rule]&.to_sym end |
#join_rule=(join_rule) ⇒ Object
Sets the join rule of the room
754 755 756 757 758 |
# File 'lib/matrix_sdk/room.rb', line 754 def join_rule=(join_rule) client.api.set_room_join_rules(id, join_rule) tinycache_adapter.write(:join_rule, join_rule) join_rule end |
#joined_members ⇒ Array(User)
Populates and returns the #members array
186 187 188 189 190 191 192 |
# File 'lib/matrix_sdk/room.rb', line 186 def joined_members client.api.get_room_joined_members(id)[:joined].map do |mxid, data| User.new(client, mxid.to_s, display_name: data.fetch(:display_name, nil), avatar_url: data.fetch(:avatar_url, nil)) end end |
#kick_user(user_id, reason = '') ⇒ Boolean
Kicks a user from the room
520 521 522 523 524 |
# File 'lib/matrix_sdk/room.rb', line 520 def kick_user(user_id, reason = '') user_id = user_id.id if user_id.is_a? MatrixSdk::User client.api.kick_user(id, user_id, reason: reason) true end |
#knock_only? ⇒ Boolean
Checks if join_rule is set to :knock
288 289 290 |
# File 'lib/matrix_sdk/room.rb', line 288 def knock_only? join_rule == :knock end |
#leave ⇒ Boolean
Requests to be removed from the room
550 551 552 553 554 |
# File 'lib/matrix_sdk/room.rb', line 550 def leave client.api.leave_room(id) client.instance_variable_get(:@rooms).delete id true end |
#members ⇒ Object
44 |
# File 'lib/matrix_sdk/room.rb', line 44 alias members joined_members |
#moderator!(user, level: 50) ⇒ Object
Make a user a moderator in the room
874 875 876 877 878 879 880 881 882 |
# File 'lib/matrix_sdk/room.rb', line 874 def moderator!(user, level: 50) return true if moderator?(user, target_level: level) user = user.id if user.is_a? User user = MXID.new(user.to_s) unless user.is_a? MXID raise ArgumentError, 'Must provide a valid user or MXID' unless user.user? modify_user_power_levels({ user.to_s.to_sym => level }) end |
#moderator?(user, target_level: 50) ⇒ Boolean
Check if a user is a moderator in the room
862 863 864 865 866 867 |
# File 'lib/matrix_sdk/room.rb', line 862 def moderator?(user, target_level: 50) level = user_powerlevel(user, use_default: false) return false unless level level >= target_level end |
#modify_required_power_levels(events = nil, params = {}) ⇒ Boolean
Modifies the required power levels for actions in the room
920 921 922 923 924 925 926 927 928 929 930 931 932 933 934 935 936 |
# File 'lib/matrix_sdk/room.rb', line 920 def modify_required_power_levels(events = nil, params = {}) return false if events.nil? && (params.nil? || params.empty?) data = power_levels_without_cache tinycache_adapter.write(:power_levels, data) data.merge!(params) data.delete_if { |_k, v| v.nil? } if events data[:events] = {} unless data.key? :events data[:events].merge!(events) data[:events].delete_if { |_k, v| v.nil? } end client.api.set_power_levels(id, data) true end |
#modify_user_power_levels(users = nil, users_default = nil) ⇒ Boolean
Modifies the power levels of the room
889 890 891 892 893 894 895 896 897 898 899 900 901 902 903 904 905 906 907 908 909 910 911 912 913 |
# File 'lib/matrix_sdk/room.rb', line 889 def modify_user_power_levels(users = nil, users_default = nil) return false if users.nil? && users_default.nil? data = power_levels_without_cache tinycache_adapter.write(:power_levels, data) data[:users_default] = users_default unless users_default.nil? if users data[:users] = {} unless data.key? :users users.each do |user, level| user = user.id if user.is_a? User user = MXID.new(user.to_s) unless user.is_a? MXID raise ArgumentError, 'Must provide a valid user or MXID' unless user.user? if level.nil? data[:users].delete(user.to_s.to_sym) else data[:users][user.to_s.to_sym] = level end end end client.api.set_power_levels(id, data) true end |
#name ⇒ String?
Will cache the current name for 15 minutes
Gets the current name of the room, querying the API if necessary
210 211 212 213 214 215 |
# File 'lib/matrix_sdk/room.rb', line 210 def name client.api.get_room_name(id)[:name] rescue MatrixNotFoundError # No room name has been specified nil end |
#name=(name) ⇒ Object
Sets a new name on the room
693 694 695 696 697 |
# File 'lib/matrix_sdk/room.rb', line 693 def name=(name) tinycache_adapter.write(:name, name) client.api.set_room_name(id, name) name end |
#power_levels ⇒ Hash
The returned power levels are cached for a minute
Get the power levels of the room
794 795 796 |
# File 'lib/matrix_sdk/room.rb', line 794 def power_levels client.api.get_power_levels(id) end |
#redact_message(event_id, reason = nil) ⇒ Object
Redacts a message from the room
464 465 466 467 |
# File 'lib/matrix_sdk/room.rb', line 464 def (event_id, reason = nil) client.api.redact_event(id, event_id, reason: reason) true end |
#reload! ⇒ Object Also known as: refresh!
Refreshes the room state caches for name, topic, and aliases
682 683 684 685 686 687 |
# File 'lib/matrix_sdk/room.rb', line 682 def reload! reload_name! reload_topic! reload_aliases! true end |
#reload_aliases! ⇒ Boolean Also known as: refresh_aliases!
The list of aliases is not sorted, ordering changes will result in alias list updates.
Reloads the list of aliases by an API query
738 739 740 |
# File 'lib/matrix_sdk/room.rb', line 738 def reload_aliases! clear_aliases_cache end |
#reload_name! ⇒ Boolean Also known as: refresh_name!
Reloads the name of the room
702 703 704 |
# File 'lib/matrix_sdk/room.rb', line 702 def reload_name! clear_name_cache end |
#reload_topic! ⇒ Boolean Also known as: refresh_topic!
Reloads the topic of the room
719 720 721 |
# File 'lib/matrix_sdk/room.rb', line 719 def reload_topic! clear_topic_cache end |
#remove_tag(tag) ⇒ Object
Remove a tag from the room
663 664 665 666 |
# File 'lib/matrix_sdk/room.rb', line 663 def remove_tag(tag) client.api.remove_user_tag(client.mxid, id, tag) true end |
#report_message(event_id, reason:, score: -100)) ⇒ Object
Reports a message in the room
474 475 476 477 |
# File 'lib/matrix_sdk/room.rb', line 474 def (event_id, reason:, score: -100) client.api.report_event(id, event_id, reason: reason, score: score) true end |
#room_type ⇒ 'm.space', ...
Retrieves the type of the room
608 609 610 611 612 613 614 |
# File 'lib/matrix_sdk/room.rb', line 608 def room_type # Can't change, so a permanent cache is ok return @room_type if @room_type_retrieved || @room_type @room_type_retrieved = true @room_type ||= creation_info[:type] end |
#room_version ⇒ String
Retrieves the room version
619 620 621 |
# File 'lib/matrix_sdk/room.rb', line 619 def room_version @room_version ||= creation_info[:room_version] end |
#send_audio(url, name, audio_info = {}) ⇒ Object
The URLs should all be of the ‘mxc://’ schema
Sends a link to an audio clip to the room
431 432 433 |
# File 'lib/matrix_sdk/room.rb', line 431 def send_audio(url, name, audio_info = {}) client.api.send_content(id, url, name, 'm.audio', extra_information: audio_info) end |
#send_custom_message(body, content = {}, msgtype: nil) ⇒ Object
Sends a customized message to the Room
440 441 442 443 444 445 446 447 |
# File 'lib/matrix_sdk/room.rb', line 440 def (body, content = {}, msgtype: nil) content.merge!( body: body, msgtype: msgtype || 'm.text' ) client.api.(id, 'm.room.message', content) end |
#send_emote(text) ⇒ Object
Sends an emote (/me) message to the room
353 354 355 |
# File 'lib/matrix_sdk/room.rb', line 353 def send_emote(text) client.api.send_emote(id, text) end |
#send_event(type, content = {}) ⇒ Object
Sends a custom timeline event to the Room
456 457 458 |
# File 'lib/matrix_sdk/room.rb', line 456 def send_event(type, content = {}) client.api.(id, type, content) end |
#send_file(url, name, file_info = {}) ⇒ Object
The URLs should all be of the ‘mxc://’ schema
Sends a link to a generic file to the room
367 368 369 |
# File 'lib/matrix_sdk/room.rb', line 367 def send_file(url, name, file_info = {}) client.api.send_content(id, url, name, 'm.file', extra_information: file_info) end |
#send_html(html, body = nil, msgtype: nil, format: nil) ⇒ Object
Sends a custom HTML message to the room
339 340 341 342 343 344 345 346 347 348 |
# File 'lib/matrix_sdk/room.rb', line 339 def send_html(html, body = nil, msgtype: nil, format: nil) content = { body: body || html.gsub(/<\/?[^>]*>/, ''), msgtype: msgtype || 'm.text', format: format || 'org.matrix.custom.html', formatted_body: html } client.api.(id, 'm.room.message', content) end |
#send_image(url, name, image_info = {}) ⇒ Object
The URLs should all be of the ‘mxc://’ schema
Sends a link to an image to the room
390 391 392 |
# File 'lib/matrix_sdk/room.rb', line 390 def send_image(url, name, image_info = {}) client.api.send_content(id, url, name, 'm.image', extra_information: image_info) end |
#send_location(geo_uri, name, thumbnail_url = nil, thumbnail_info = {}) ⇒ Object
The thumbnail URL should be of the ‘mxc://’ schema
Sends a location object to the room
401 402 403 |
# File 'lib/matrix_sdk/room.rb', line 401 def send_location(geo_uri, name, thumbnail_url = nil, thumbnail_info = {}) client.api.send_location(id, geo_uri, name, thumbnail_url: thumbnail_url, thumbnail_info: thumbnail_info) end |
#send_notice(text) ⇒ Object
Sends a notice (bot) message to the room
374 375 376 |
# File 'lib/matrix_sdk/room.rb', line 374 def send_notice(text) client.api.send_notice(id, text) end |
#send_text(text) ⇒ Object
Sends a plain-text message to the room
326 327 328 |
# File 'lib/matrix_sdk/room.rb', line 326 def send_text(text) client.api.(id, text) end |
#send_video(url, name, video_info = {}) ⇒ Object
The URLs should all be of the ‘mxc://’ schema
Sends a link to a video to the room
418 419 420 |
# File 'lib/matrix_sdk/room.rb', line 418 def send_video(url, name, video_info = {}) client.api.send_content(id, url, name, 'm.video', extra_information: video_info) end |
#set_account_data(type, account_data) ⇒ Object
Stores a custom entry into the room-specific account data
574 575 576 577 |
# File 'lib/matrix_sdk/room.rb', line 574 def set_account_data(type, account_data) self.account_data[type] = account_data true end |
#set_user_profile(display_name: nil, avatar_url: nil, reason: nil) ⇒ Object
the avatar URL should be a mxc:// URI
Changes the room-specific user profile
584 585 586 587 588 589 590 591 592 593 594 595 |
# File 'lib/matrix_sdk/room.rb', line 584 def set_user_profile(display_name: nil, avatar_url: nil, reason: nil) return nil unless display_name || avatar_url data = client.api.get_membership(id, client.mxid) raise "Can't set profile if you haven't joined the room" unless data[:membership] == 'join' data[:displayname] = display_name unless display_name.nil? data[:avatar_url] = avatar_url unless avatar_url.nil? client.api.set_membership(id, client.mxid, 'join', reason || 'Updating room profile information', data) true end |
#space? ⇒ Boolean?
Checks if the room is a Matrix Space
626 627 628 629 630 |
# File 'lib/matrix_sdk/room.rb', line 626 def space? room_type == 'm.space' rescue MatrixSdk::MatrixForbiddenError, MatrixSdk::MatrixNotFoundError nil end |
#tags ⇒ Response
Returns a list of the room tags
642 643 644 645 646 647 648 649 650 651 652 653 654 655 656 657 658 |
# File 'lib/matrix_sdk/room.rb', line 642 def client.api.(client.mxid, id)[:tags].tap do |tag_obj| tag_obj.instance_variable_set(:@room, self) tag_obj.define_singleton_method(:room) do @room end tag_obj.define_singleton_method(:add) do |tag, **data| @room.add_tag(tag.to_s.to_sym, **data) self[tag.to_s.to_sym] = data self end tag_obj.define_singleton_method(:remove) do |tag| @room.remove_tag(tag.to_s.to_sym) delete tag.to_s.to_sym end end end |
#to_s ⇒ Object
112 113 114 115 116 117 118 |
# File 'lib/matrix_sdk/room.rb', line 112 def to_s prefix = canonical_alias if canonical_alias_has_value? prefix ||= id return "#{prefix} | #{name}" if name_has_value? prefix end |
#to_space ⇒ Object
Casting operators
106 107 108 109 110 |
# File 'lib/matrix_sdk/room.rb', line 106 def to_space return nil unless space? Rooms::Space.new self, nil end |
#topic ⇒ String?
Gets the room topic - if any
256 257 258 259 260 261 |
# File 'lib/matrix_sdk/room.rb', line 256 def topic client.api.get_room_topic(id)[:topic] rescue MatrixNotFoundError # No room name has been specified nil end |
#topic=(topic) ⇒ Object
Sets a new topic on the room
710 711 712 713 714 |
# File 'lib/matrix_sdk/room.rb', line 710 def topic=(topic) tinycache_adapter.write(:topic, topic) client.api.set_room_topic(id, topic) topic end |
#unban_user(user_id) ⇒ Boolean
Unbans a user from the room
541 542 543 544 545 |
# File 'lib/matrix_sdk/room.rb', line 541 def unban_user(user_id) user_id = user_id.id if user_id.is_a? MatrixSdk::User client.api.unban_user(id, user_id) true end |
#user_can_send?(user, event, state: false) ⇒ Boolean
Checks if a user can send a given event type in the room
820 821 822 823 824 825 826 |
# File 'lib/matrix_sdk/room.rb', line 820 def user_can_send?(user, event, state: false) user_pl = user_powerlevel(user) event_pl = power_levels.dig(:events, event.to_s.to_sym) event_pl ||= state ? (power_levels[:state_default] || 50) : (power_levels[:events_default] || 0) user_pl >= event_pl end |
#user_powerlevel(user, use_default: true) ⇒ Integer?
Gets the power level of a user in the room
804 805 806 807 808 809 810 811 812 |
# File 'lib/matrix_sdk/room.rb', line 804 def user_powerlevel(user, use_default: true) user = user.id if user.is_a? User user = MXID.new(user.to_s) unless user.is_a? MXID raise ArgumentError, 'Must provide a valid User or MXID' unless user.user? level = power_levels.dig(:users, user.to_s.to_sym) level ||= power_levels[:users_default] || 0 if use_default level end |
#world_readable? ⇒ Boolean Also known as: world_readable
Checks if the room history is world readable
302 303 304 |
# File 'lib/matrix_sdk/room.rb', line 302 def world_readable? history_visibility == :world_readable end |